@@ -1,9 +1,6 @@
-# -*- python -*-
-# -*- coding: utf-8 -*-
-#
 #  This file is part of the fitter software
 #
-#  Copyright (c) 2014
+#  Copyright (c) 2014-2022
 #
 #  File author(s): Thomas Cokelaer <cokelaer@gmail.com>
 #
@@ -21,24 +18,62 @@
 .. sectionauthor:: Thomas Cokelaer, Aug 2014-2020
 
 """
-import logging
 import sys
+import contextlib
 import threading
 from datetime import datetime
 
+import logging
+
 import numpy as np
 import pandas as pd
 import pylab
 import scipy.stats
-from easydev import Progress
-from joblib import Parallel, delayed
-from scipy.stats import entropy as kl_div
+
+import joblib
+from joblib.parallel import Parallel, delayed
+from tqdm import tqdm
+from scipy.stats import entropy as kl_div, kstest
 
 logger = logging.getLogger(__name__)
 
 __all__ = ["get_common_distributions", "get_distributions", "Fitter"]
 
 
+
+# A solution to wrap joblib parallel call in tqdm from 
+# https://stackoverflow.com/questions/24983493/tracking-progress-of-joblib-parallel-execution/58936697#58936697
+# and https://github.com/louisabraham/tqdm_joblib
+@contextlib.contextmanager
+def tqdm_joblib(*args, **kwargs):
+    """Context manager to patch joblib to report into tqdm progress bar
+    given as argument"""
+
+    tqdm_object = tqdm(*args, **kwargs)
+
+    class TqdmBatchCompletionCallback(joblib.parallel.BatchCompletionCallBack):
+        def __init__(self, *args, **kwargs):
+            super().__init__(*args, **kwargs)
+
+        def __call__(self, *args, **kwargs):
+            tqdm_object.update(n=self.batch_size)
+            return super().__call__(*args, **kwargs)
+
+    old_batch_callback = joblib.parallel.BatchCompletionCallBack
+    joblib.parallel.BatchCompletionCallBack = TqdmBatchCompletionCallback
+    try:
+        yield tqdm_object
+    finally:
+        joblib.parallel.BatchCompletionCallBack = old_batch_callback
+        tqdm_object.close()
+
+
+
+
+
+
+
+
 def get_distributions():
     distributions = []
     for this in dir(scipy.stats):
@@ -113,7 +148,7 @@ class Fitter(object):
     :attr:`distributions` with a subset if you want. In order to reload all distributions,
     call :meth:`load_all_distributions`.
 
-    Some distributions do not converge when fitting. There is a timeout of 10 seconds after which
+    Some distributions do not converge when fitting. There is a timeout of 30 seconds after which
     the fitting procedure is cancelled. You can change this :attr:`timeout` attribute if needed.
 
     If the histogram of the data has outlier of very long tails, you may want to increase the
@@ -199,8 +234,10 @@ class Fitter(object):
         self._aic = {}
         self._bic = {}
         self._kldiv = {}
+        self._ks_stat = {}
+        self._ks_pval = {}
         self._fit_i = 0  # fit progress
-        self.pb = None
+        #self.pb = None
 
     def _update_data_pdf(self):
         # histogram retuns X with N+1 values. So, we rearrange the X output into only N
@@ -208,9 +245,7 @@ class Fitter(object):
         self.x = [(this + self.x[i + 1]) / 2.0 for i, this in enumerate(self.x[0:-1])]
 
     def _trim_data(self):
-        self._data = self._alldata[
-            np.logical_and(self._alldata >= self._xmin, self._alldata <= self._xmax)
-        ]
+        self._data = self._alldata[np.logical_and(self._alldata >= self._xmin, self._alldata <= self._xmax)]
 
     def _get_xmin(self):
         return self._xmin
@@ -224,9 +259,7 @@ class Fitter(object):
         self._trim_data()
         self._update_data_pdf()
 
-    xmin = property(
-        _get_xmin, _set_xmin, doc="consider only data above xmin. reset if None"
-    )
+    xmin = property(_get_xmin, _set_xmin, doc="consider only data above xmin. reset if None")
 
     def _get_xmax(self):
         return self._xmax
@@ -240,9 +273,7 @@ class Fitter(object):
         self._trim_data()
         self._update_data_pdf()
 
-    xmax = property(
-        _get_xmax, _set_xmax, doc="consider only data below xmax. reset if None "
-    )
+    xmax = property(_get_xmax, _set_xmax, doc="consider only data below xmax. reset if None ")
 
     def _load_all_distributions(self):
         """Replace the :attr:`distributions` attribute with all scipy distributions"""
@@ -264,7 +295,7 @@ class Fitter(object):
         _ = pylab.hist(self._data, bins=self.bins, density=self._density)
         pylab.grid(True)
 
-    def _fit_single_distribution(self, distribution, progress: bool):
+    def _fit_single_distribution(self, distribution):
         try:
             # need a subprocess to check time it takes. If too long, skip it
             dist = eval("scipy.stats." + distribution)
@@ -287,41 +318,38 @@ class Fitter(object):
             # calculate error
             sq_error = pylab.sum((self.fitted_pdf[distribution] - self.y) ** 2)
 
-            # calcualte information criteria
+            # calculate information criteria
             logLik = np.sum(dist.logpdf(self.x, *param))
             k = len(param[:])
             n = len(self._data)
             aic = 2 * k - 2 * logLik
             bic = n * np.log(sq_error / n) + k * np.log(n)
 
-            # calcualte kullback leibler divergence
+            # calculate kullback leibler divergence
             kullback_leibler = kl_div(self.fitted_pdf[distribution], self.y)
 
-            logging.info(
-                "Fitted {} distribution with error={})".format(distribution, sq_error)
-            )
+            # calculate goodness-of-fit statistic
+            dist_fitted = dist(*param)
+            ks_stat, ks_pval = kstest(self._data, dist_fitted.cdf)
+
+            logging.info("Fitted {} distribution with error={})".format(distribution, sq_error))
 
             # compute some errors now
             self._fitted_errors[distribution] = sq_error
             self._aic[distribution] = aic
             self._bic[distribution] = bic
             self._kldiv[distribution] = kullback_leibler
+            self._ks_stat[distribution] = ks_stat
+            self._ks_pval[distribution] = ks_pval
         except Exception:  # pragma: no cover
-            logging.warning(
-                "SKIPPED {} distribution (taking more than {} seconds)".format(
-                    distribution, self.timeout
-                )
-            )
+            logging.warning("SKIPPED {} distribution (taking more than {} seconds)".format(distribution, self.timeout))
             # if we cannot compute the error, set it to large values
             self._fitted_errors[distribution] = np.inf
             self._aic[distribution] = np.inf
             self._bic[distribution] = np.inf
             self._kldiv[distribution] = np.inf
-        if progress:
-            self._fit_i += 1
-            self.pb.animate(self._fit_i)
 
-    def fit(self, amp=1, progress=False, n_jobs=-1):
+    def fit(self, progress=False, n_jobs=-1, max_workers=-1):
         r"""Loop over distributions and find best parameter to fit the data for each
 
         When a distribution is fitted onto the data, we populate a set of
@@ -339,21 +367,19 @@ class Fitter(object):
 
         warnings.filterwarnings("ignore", category=RuntimeWarning)
 
-        if progress:
-            self.pb = Progress(len(self.distributions))
+        N = len(self.distributions)
+        with tqdm_joblib(desc=f"Fitting {N} distributions", total=N) as progress_bar:
+            Parallel(n_jobs=max_workers, backend='threading')(delayed(self._fit_single_distribution)(dist) for dist in self.distributions)
+
 
-        jobs = (
-            delayed(self._fit_single_distribution)(dist, progress)
-            for dist in self.distributions
-        )
-        pool = Parallel(n_jobs=n_jobs, backend="threading")
-        _ = pool(jobs)
         self.df_errors = pd.DataFrame(
             {
                 "sumsquare_error": self._fitted_errors,
                 "aic": self._aic,
                 "bic": self._bic,
                 "kl_div": self._kldiv,
+                "ks_statistic": self._ks_stat,
+                "ks_pvalue": self._ks_pval,
             }
         )
 
@@ -399,11 +425,7 @@ class Fitter(object):
         name = self.df_errors.sort_values(method).iloc[0].name
         params = self.fitted_param[name]
         distribution = getattr(scipy.stats, name)
-        param_names = (
-            (distribution.shapes + ", loc, scale").split(", ")
-            if distribution.shapes
-            else ["loc", "scale"]
-        )
+        param_names = (distribution.shapes + ", loc, scale").split(", ") if distribution.shapes else ["loc", "scale"]
 
         param_dict = {}
         for d_key, d_val in zip(param_names, params):
@@ -456,9 +478,7 @@ class Fitter(object):
         ended_at = datetime.now()
         diff = ended_at - started_at
 
-        if (
-            it.exc_info[0] is not None
-        ):  # pragma: no cover ;  if there were any exceptions
+        if it.exc_info[0] is not None:  # pragma: no cover ;  if there were any exceptions
             a, b, c = it.exc_info
             raise Exception(a, b, c)  # communicate that to caller
 
